#pragma once
#include <iostream>
#include <memory>
#include <string>
#include <unordered_map>

#include "likeness_map.h"

class flood_detector
	{
public:
	static std::shared_ptr<flood_detector> instance()
		{
		if ( ! flood_detector::instance_ )
			{
			flood_detector::instance_.reset(new flood_detector());
			}
		return flood_detector::instance_;
		}

	likeness_map::ptr_t new_source(const std::string& source)
		{
		auto it = source_map_.find(source);
		if ( it != source_map_.end() )
			{
			std::cerr << "WARNING: source map exists for source: " << source << "\n";
			source_map_.erase(it);
			}
		auto inserted = source_map_.insert(
			std::make_pair(source, std::make_shared<likeness_map>()));
		if ( inserted.second )
			return inserted.first->second;
		return nullptr;
		}

	bool delete_source(const std::string& source)
		{
		auto it = source_map_.find(source);
		if ( it == source_map_.end() )
			return false;
		source_map_.erase(it);
		return true;
		}

	likeness_map::ptr_t find(const std::string& source)
		{
		auto it = source_map_.find(source);
		if ( it == source_map_.end() )
			{
			std::cerr << "WARNING: source map not found for source: " << source << "\n";
			return new_source(source);
			}
		return it->second;
		}

	void count_entry(const std::string& source)
		{
		auto lm = find(source);
		if ( lm )
			lm->count();
		}

	bool should_warn(const std::string& source, const std::string& msg)
		{
		auto lm = find(source);
		uint64_t warning_count = 0;
		if ( lm )
			warning_count = lm->occurrence(msg);
		auto ratio = lm->ratio(msg);
		if ( warning_count > 256 && ratio > 0.5 )
			return warning_count % 10000 == 0;
		return true;
		}

private:
	static std::shared_ptr<flood_detector> instance_;
	std::unordered_map<std::string, likeness_map::ptr_t> source_map_;
	};

std::shared_ptr<flood_detector> flood_detector::instance_ = 0;
