
#pragma once
#include <regex>
#include <sstream>
#include <string>
#include <unordered_map>

class likeness_map
	{
public:
	typedef std::shared_ptr<likeness_map> ptr_t;

	likeness_map() : call_count_(0) { }
	void count()
		{
		call_count_++;
		}
	uint64_t occurrence(const std::string& msg)
		{
		auto phrase = only_words(msg);
		auto it = phrase_map_.find(phrase);
		if ( it == phrase_map_.end() )
			{
			phrase_map_[phrase] = 1;
			}
		else
			{
			phrase_map_[phrase]++;
			}
		return phrase_map_[phrase];
		}

	double ratio(const std::string& msg)
		{
		auto phrase = only_words(msg);
		auto it = phrase_map_.find(phrase);
		if ( it == phrase_map_.end() )
			return 0.0;
		return static_cast<double>(it->second) / static_cast<double>(call_count_);
		}

	std::string only_words(const std::string& input_str)
		{
		std::regex r("([.0-9 :-]+)");
		std::vector<std::string> tokens(
			std::sregex_token_iterator(input_str.begin(), input_str.end(), r, -1),
			std::sregex_token_iterator());
		std::ostringstream output_str;
		std::copy(tokens.begin(), tokens.end(),
		          std::ostream_iterator<std::string>(output_str, " "));
		return output_str.str();
		}

private:
	static std::shared_ptr<likeness_map> instance_;
	std::unordered_map<std::string, uint64_t> phrase_map_;
	uint64_t call_count_;
	};

std::shared_ptr<likeness_map> likeness_map::instance_ = 0;
